;;; gpl.el --- shortcut commands for inserting copyright notices
;;
;; Copyright (C) 1998 Eleftherios Gkioulekas <lf@amath.washington.edu>
;; This file is part of GNU Emacs.
;;
;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to the
;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; To use the facilities provided in this file you need to add the following
;; to your .emacs file:
;; a. You must tell Emacs who you are
;;    (setq user-mail-address "lf@amath.washington.edu")
;;    (setq user-full-name    "Homer Simpson")
;; b. You must require that this file is loaded
;;    (require 'gpl)
;;

(defconst gpl-copyright-line 
  (concat "Copyright (C) "
          (format-time-string "%Y" (current-time)) 
          " " user-full-name " <" user-mail-address ">"))

;; The text of the generic header
(defconst gpl-generic-notice
 '(" "
   "This program is free software; you can redistribute it and/or modify"
   "it under the terms of the GNU General Public License as published by"
   "the Free Software Foundation; either version 2 of the License, or"
   "(at your option) any later version."
   " "
   "This program is distributed in the hope that it will be useful,"
   "but WITHOUT ANY WARRANTY; without even the implied warranty of"
   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the"
   "GNU General Public License for more details."
   " "
   "You should have received a copy of the GNU General Public License"
   "along with this program; if not, write to the Free Software"
   "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA."
   " " ))

;; The text of the Autoconf header
(defconst gpl-autoconf-permission-notice
  '(" "
    "As a special exception to the GNU General Public License, if you"
    "distribute this file as part of a program that contains a configuration"
    "script generated by Autoconf, you may include it under the same"
    "distribution terms that you use for the rest of that program."
    " "))

;; Notice for Texinfo files
(defconst gpl-texinfo-notice
  '(" "
    "Permission is granted to make and distribute verbatim copies of"
    "this manual provided the copyright notice and this permission notice"
    "are preserved on all copies."
    " "
    "Permission is granted to process this file through TeX and print the"
    "results, provided the printed document carries copying permission"
    "notice identical to this one except for the removal of this paragraph"
    "(this paragraph not being relevant to the printed manual)."
    " "
    "Permission is granted to copy and distribute modified versions of this"
    "manual under the conditions for verbatim copying, provided that the"
    "entire resulting derived work is distributed under the terms of a "
    "permission notice identical to this one."
    " "
    "Permission is granted to copy and distribute translations of this manual"
    "into another language, under the above conditions for modified versions,"
    "except that this permission notice may be stated in a translation"
    "approved by the Free Software Foundation"
    " "))

;; The text of the Guile-lisk permission notice.
(defun gpl-library-notice (name)
 "This function returns a library copyright notice for a specific library
called `name'. This notice is fashioned after the GUILE notice, and it
is meant to follow the standard GPL notice."
 (list
  (concat "As a special exception to the GNU General Public License,")
  (concat "permission is granted for additional uses of the text contained in")
  (concat "its release of " name ".")
  (concat " ")
  (concat "The exception is that, if you link the " name " library with other")
  (concat "files to produce an executable, this does not by itself cause the")
  (concat "resulting executable to be covered by the GNU General Public License")
  (concat " ")
  (concat "This exception applies only to the code released under the name")
  (concat name ". If you copy code from other releases into a copy of")
  (concat name ", as the General Public License permits, the exception does")
  (concat "not apply to the code that you add in this way. To avoid")
  (concat "misleading anyone as to the status of such modified files, you")
  (concat "must delete this exception notice from them.")
  (concat " ")
  (concat "If you write modifications of your own for " name ", it is your")
  (concat "choice whether to permit this exception to apply to your")
  (concat "modifications. If you do not wish that, delete this exception notice.")
  (concat " ")))

;; The text of the special header
;; The text of the m4 header extra permission
;; The guile header

;; Insert a header with appropriate commenting
(defun gpl-insert-notice (comment notice)
  "This function inserts a copyright notice stored in list `notice'
prepending it with the comment sequence `comment' and terminating each line
with a new line"
  (while notice
    (insert comment " " (car notice) "\n")
    (setq notice (cdr notice))))

;;;
;;; User level commands for ordinary gpl notices
;;;

;; 
;; Commands for C programs
;;

(defun gpl-c-copyright-line ()
  "This function is a helper function to be used by the functions
gpl-c, gpl-cL. It inserts the copyright line"
  (insert "/*\n" "** " gpl-copyright-line "\n" ))

(defun gpl-c ()
  "This function inserts a standard GPL copyright notice using C commenting."
  (interactive)
  (gpl-c-copyright-line)
  (gpl-insert-notice "**" gpl-generic-notice)
  (insert "*/\n"))

(defun gpl-cL ()
  "This function inserts a standard GPL copyright notice using C commenting,
followed by a Guile-like library exception."
  (interactive)
  (gpl-c-copyright-line)
  (gpl-insert-notice "**" gpl-generic-notice)
  (gpl-insert-notice "**" (gpl-library-notice (read-string "Library name: ")))
  (insert "*/\n"))

;;
;; Commands for C++ programs
;;

(defun gpl-cc-copyright-line ()
  "This function is a helper function to be used by the functions
gpl-cc, gpl-ccL. It inserts the copyright line"
  (insert "// " gpl-copyright-line "\n"))

(defun gpl-cc ()
  "This function inserts a standard GPL copyright notice using C++ commenting."
  (interactive)
  (gpl-cc-copyright-line)
  (gpl-insert-notice "//" gpl-generic-notice))

(defun gpl-ccL ()
  "This function inserts a standard GPL copyright notice using C++ commenting,
followed by a Guile-like library exception."
  (interactive)
  (gpl-cc-copyright-line)
  (gpl-insert-notice "//" gpl-generic-notice)
  (gpl-insert-notice "//" (gpl-library-notice (read-string "Library name: "))))

;;
;; Commands for shell scripts and such
;;

(defun gpl-sh ()
  "This function inserts a standard GPL copyright notice using shell
commenting (i.e. hash marks)"
  (interactive)
  (insert "# " gpl-copyright-line "\n")
  (gpl-insert-notice "#" gpl-generic-notice))

(defun gpl-shL ()
  "This function inserts a standard GPL copyright notice using shell
commenting, followed by a Guile-like library exception. This can be
useful for things like Tcl scripts example."
  (interactive)
  (insert "# " gpl-copyright-line "\n")
  (gpl-insert-notice "#" gpl-generic-notice)
  (gpl-insert-notice "#" (gpl-library-notice (read-string "Library name: "))))

(defun gpl-shS ()
  "This function inserts a standard GPL copyright notice using shell
commenting, followed by the special Autoconf exception. This is useful for
shell scripts that are distributed as part of a build system."
  (interactive) 
  (insert "# " gpl-copyright-line "\n")
  (gpl-insert-notice "#" gpl-generic-notice)
  (gpl-insert-notice "#" gpl-autoconf-permission-notice))


;;
;; Command for m4 macros
;;

(defun gpl-m4 ()
  "This function inserts the standard GPL copyright notice using m4
commenting (i.e. dnl) and the special Autoconf exception. This is useful
for Autoconf macros."
  (interactive)
  (insert "dnl " gpl-copyright-line "\n")
  (gpl-insert-notice "dnl" gpl-generic-notice)
  (gpl-insert-notice "dnl" gpl-autoconf-permission-notice))

;;
;; Command for elisp files
;;

(defun gpl-el ()
  "This function inserts the standard GPL copyright notice using elisp
commenting. This is useful for elisp files."
  (interactive)
  (insert ";;; " gpl-copyright-line "\n")
  (gpl-insert-notice ";;;" gpl-generic-notice))

;; foooooo
(defun gpl-test()
 "This is a test"
 (interactive)
 (insert "** " gpl-copyright-line "\n")
 (gpl-insert-notice "**" (gpl-library-notice (read-string "Library name: "))))

;;
;; Command for Texinfo files
;;

(defun gpl-texinfo ()
  "Insert a standard Texinfo copyright notice using Texinfo commenting."
  (interactive)
  (insert "@c " gpl-copyright-line "\n")
  (gpl-insert-notice "@c" gpl-texinfo-notice))

(provide 'gpl)
